@@ -24,9 +24,29 @@ module Async
 	# Represents an asynchronous IO within a reactor.
 	class Wrapper
 		class Cancelled < StandardError
+			class From
+				def initialize
+					@backtrace = caller[5..-1]
+				end
+				
+				attr :backtrace
+				
+				def cause
+					nil
+				end
+				
+				def message
+					"Cancelled"
+				end
+			end
+			
 			def initialize
 				super "The operation has been cancelled!"
+				
+				@cause = From.new
 			end
+			
+			attr :cause
 		end
 		
 		# wait_readable, wait_writable and wait_any are not re-entrant, and will raise this failure.
@@ -92,14 +112,14 @@ module Async
 		end
 		
 		# Wait for the io to become readable.
-		def wait_readable(duration = nil)
+		def wait_readable(timeout = nil)
 			raise WaitError if @readable
 			
 			self.reactor = Task.current.reactor
 			
 			begin
 				@readable = Fiber.current
-				wait_for(duration)
+				wait_for(timeout)
 			ensure
 				@readable = nil
 				@monitor.interests = interests if @monitor
@@ -107,14 +127,14 @@ module Async
 		end
 		
 		# Wait for the io to become writable.
-		def wait_writable(duration = nil)
+		def wait_writable(timeout = nil)
 			raise WaitError if @writable
 			
 			self.reactor = Task.current.reactor
 			
 			begin
 				@writable = Fiber.current
-				wait_for(duration)
+				wait_for(timeout)
 			ensure
 				@writable = nil
 				@monitor.interests = interests if @monitor
@@ -123,14 +143,14 @@ module Async
 		
 		# Wait fo the io to become either readable or writable.
 		# @param duration [Float] timeout after the given duration if not `nil`.
-		def wait_any(duration = nil)
+		def wait_any(timeout = nil)
 			raise WaitError if @any
 			
 			self.reactor = Task.current.reactor
 			
 			begin
 				@any = Fiber.current
-				wait_for(duration)
+				wait_for(timeout)
 			ensure
 				@any = nil
 				@monitor.interests = interests if @monitor
@@ -195,7 +215,7 @@ module Async
 			end
 		end
 		
-		def wait_for(duration)
+		def wait_for(timeout)
 			if @monitor
 				@monitor.interests = interests
 			else
@@ -203,13 +223,9 @@ module Async
 			end
 			
 			# If the user requested an explicit timeout for this operation:
-			if duration
-				@reactor.timeout(duration) do
-					begin
-						Task.yield
-					rescue Async::TimeoutError
-						return false
-					end
+			if timeout
+				@reactor.with_timeout(timeout) do
+					Task.yield
 				end
 			else
 				Task.yield
